## Changed script so it depends on the included ZDV data set only
## Batch arguments and variable selection is now removed
## No more dependencies on hivAnalyzer and its data

## Set parameters
	outputPath <- TEMP		# Output path
	set.seed(4242)				# Seed
	K <- 10								# Number of folds in CV
	epsGuess <- 0.05			# allowed violations for an edge to be included
	epsStop <- 1					# stop EM if likelihood difference below epsStop
	nIter <- 1						# Number of iterations in the annealing run
	tInit <- 100					# Initial temperature in the annealing run
	tAlpha <- 0.1					# Temperature for preselecting propositions in the annealing run
  tFactor <- 1					# cooling factor in the annealing run
  maxIter <- 50					# Maximal number of iterations in isotone
	
## Load data
	library(icbn)
	data(zdv)
	cat("outputPath:", outputPath, "\n")
  Y <- zdv[,1]
	X <- zdv[,-1]
	xStat <- table(apply(X, 1, function(x){ paste(x, collapse="") }))
	p <- ncol(X)
	
## Assign poset either by annealing or by compatibility check
#	P <- hasse(constructPoset(hyperCube(p), map2HyperCube(X, Y), eps=epsGuess)$poset[[1]])
	P <- doAnneal(X, Y, outputPath, tInit=tInit, nIter=nIter, loggingFlag=FALSE, tAlpha=tAlpha,
                epsStop=epsStop, epsGuess=epsGuess, tFactor=tFactor, maxIter=maxIter)$P

## Cross-validation part
	cvResult <- cvIcbn(fitCbn, X, Y, K, P, epsStop=epsStop)
	pValMse <- wilcox.test(cvResult$mseAllLr, cvResult$mseAllIcbn, paired=FALSE, alternative="greater")$p.value
	pValCor <- wilcox.test(cvResult$corAllLr, cvResult$corAllIcbn, paired=FALSE, alternative="less")$p.value
	pValMseP <- wilcox.test(cvResult$mseAllLr, cvResult$mseAllIcbn, paired=TRUE, alternative="greater")$p.value
	pValCorP <- wilcox.test(cvResult$corAllLr, cvResult$corAllIcbn, paired=TRUE, alternative="less")$p.value
	results <- list(cvResult=cvResult, xStat=xStat, P=P, X=X, Y=Y,
									pValMse=pValMse, pValCor=pValCor, pValMseP=pValMseP, pValCorP=pValCorP)

## Train model using the fitted poset from the annealing run
	parIcbn <- guessParams(X, Y, P)
	parIcbn <- doEm(X, Y, parIcbn, eps=epsStop, maxIter=maxIter)
	printParams(parIcbn, "Fitted parameters on 100% of the data")
					
## Predict mus for ICBN and LR
	lrMod <- lm(Y~., data=X)
	parLr		<- lrMod$coef
	perfTest <- predictPheno(X, Y, parIcbn, parLr)
	xHyper <- as.data.frame(parIcbn$H)
	colnames(xHyper) <- colnames(X) 
	musLr <- predict(lrMod, xHyper)[as.logical(parIcbn$G)]

## Count and extract mean mus from data	
	musData <-  rep(NA, length(musLr))
	countData <- rep(0, length(musLr))
	names(musData) <- names(countData) <- names(musLr)
	names(Y) <- apply(X, 1, function(x) paste(x, collapse=""))
	musDataAll <- tapply(Y, names(Y), FUN=mean)
	countDataAll <- tapply(Y, names(Y), FUN=length)
	for (i in 1:length(musData)){
		idx <- which(names(musDataAll)==names(musData[i]))
		if (length(idx) > 0){
			musData[i] <- musDataAll[idx]
			countData[i] <- countDataAll[idx]
		}
	}

## Stats for the plot
	nonLatticeCount <- sum(countDataAll[which(names(countDataAll) %in% setdiff(names(countDataAll),names(countData)))])
  inLatticeCount <-  sum(countDataAll[which(names(countDataAll) %in% intersect(names(countDataAll),names(countData)))])
	nonLatticeMean <- mean(musDataAll[which(names(countDataAll) %in% setdiff(names(countDataAll),names(countData)))])
	inLatticeMean <-  mean(musDataAll[which(names(countDataAll) %in% intersect(names(countDataAll),names(countData)))])

## Save workspace
	save.image(file=paste(outputPath, "result_ZDV.Rdata", sep=""))

## Plot both models, simulated and fitted
	pdf(file=paste(outputPath, "plot_ZDV.pdf", sep=""), width=12, height=9)
		par(mfrow=c(1,1), cex.sub=0.7, cex.main=0.8, cex.lab=0.7)
		ug <- round(colSums(parIcbn$respMat),0)
		plotPoset(P, vertexLabels=paste(colnames(X), round(parIcbn$thetas, 2), sep="\n"), main=paste("ZDV, logLikObs:", round(parIcbn$lObs,2), sep=""),
							sub=paste("Fitted epsilon: ", round(parIcbn$epsilon,2), ", fitted sigma^2: ", round(parIcbn$sigma2,2), 
							"\nFitted thetas: vertex labels",
							"\npVal(MSE LR > MSE ICBN, paired): ", round(pValMseP, 4), ", pVal(COR LR < COR ICBN, paired): ", round(pValCorP, 4),
							sep=""), vertex.size=20, edge.arrow.size=0.8, vertex.label.cex=0.6)
		plotIsotonic(parIcbn$mus, parIcbn$G, parIcbn$H, colnames(X), ug, layout="reingold", 
							sub=paste(
							"Vertex labels: genotype, fitted expected count, fitted mu at genotype with CBN model.\n",
							K, "-fold cross-validation\n", 
							"MSE(LR): ", round(cvResult$mseLr, 2), "(", round(cvResult$seMseLr, 3), ")", 
							", MSE(CBN): ", round(cvResult$mseIcbn, 2), "(", round(cvResult$seMseIcbn, 3), ")\n",
							"COR(LR): ", round(cvResult$corLr, 2), "(", round(cvResult$seCorLr, 3), ")",
							", COR(CBN): ", round(cvResult$corIcbn, 2), "(", round(cvResult$seCorIcbn, 3), ")\n",
							sep=""), vertex.size=10, edge.arrow.size=0.2, vertex.label.cex=0.6, vertex.shape="square")
		plotIsotonic(musLr, parIcbn$G, parIcbn$H, colnames(X), rep(NULL, length(ug)), layout="reingold", 
							sub=paste(
						  "Vertex labels: genotype, fitted mu at genotype with linear model.\n",
							sep=""), vertex.size=20, edge.arrow.size=0.2, vertex.label.cex=0.6)
		 plotIsotonic(musData, parIcbn$G, parIcbn$H, colnames(X), countData, layout="reingold", 
							sub=paste(
							"Vertex labels: genotype, sample count, average mu at genotype\n",
							"Observations in lattice: ", inLatticeCount, ", observations out of lattice: ", nonLatticeCount, "\n",
							"Mean phenotype in lattice: ", round(inLatticeMean,2), ", mean phenotype out of lattice: ", round(nonLatticeMean,2), 
							sep=""), vertex.size=20, edge.arrow.size=0.2, vertex.label.cex=0.6)
		plot(perfTest$y, perfTest$yhatIcbn, col="green", xlab="Observed value", ylab="Predicted value", 
			main=paste("Predicted vs. observed y-values\nDrug: ", "ZDV", sep=""), pch=16)
		points(perfTest$y, perfTest$yhatLr, col="red", pch=17)
		abline(0,1)
		legend("topleft", c("Icbn", "Lr"), col=c("green", "red"), pch=c(16,17), cex=0.7)
		plot(1:length(perfTest$y), (perfTest$y), xlab="Index", ylab="Predicted value", main="y-values in data order", pch=18, col="black")
		points(1:length(perfTest$y), (perfTest$yhatLr), col="red", pch=17)
		points(1:length(perfTest$y), (perfTest$yhatIcbn), col="green", pch=16)
		plot(sort(perfTest$y), col="black", main="fitted values", sub="horizontal lines correspond\nto the mus fitted", pch=18, xlab="",
						ylim=c(-2, 4), ylab="")
		points(perfTest$yhatLr[order(perfTest$y)], col="red", pch=17)
		points(perfTest$yhatIcbn[order(perfTest$y)], col="green", pch=16)
		abline(h=parIcbn$mus, lty=2)
		legend("bottomright", c("True", "Icbn", "Lr"), col=c("black", "green", "red"), pch=c(18, 16, 17), cex=0.7)
	dev.off()

## End
