## Draws random samples from the NI-CBN and fits the generated data
## with simulated annealing
## A run for a given poset calculates 12 different configurations
## epsilon: 0.01, 0.001, 0.1
## sigma^2: 1, 0.1
## Number of records: 500, 1000

## Set Parameters
	nP <- 5  									# Number of the poset, 1 to 6
	nR <- 1										# Number of runs
	outputPath <- TEMP				# Output path
	seed <- 1234							# Random number seed
	structureSearch <- TRUE		# Flag to set for a annealing structure search
	K <- 10										# k-fold CV, number of folds
	epsGuess <- 0.05					# allowed violations for an edge to be included
	epsStop <- 1							# stop EM if likelihood difference below epsStop
	nIter <- 10								# Number of iterations in the annealing run
	tInit <- 100							# Initial temperature in the annealing run
	tAlpha <- 0.1							# Temperature for preselecting propositions in the annealing run
  tFactor <- 1							# cooling factor in the annealing run
  maxIter <- 50							# Maximal number of iterations in isotone

## Print out parameters
  cat("Simulating poset:", nP, "\n")
  cat("Number of runs:", nR, "\n")
  cat("Output path:", outputPath, "\n")
  cat("Initial seed:" , seed, "\n")
  cat("Structure search:", structureSearch, "\n")

## Init and load ZDV data
	library(icbn)
	set.seed(seed)
	for (run in 1:nR){
		for (nE in 1:3){
			for (nS in 1:2){
				for (nN in 1:2){
		
				## Lists of all possibilities
					posetList <- list(
					  matrix(0, 4, 4),
						matrix(0, 7, 7),
						matrix(c(0,1,0,0,0,0,1,0,0,0,0,1,0,0,0,0), nc=4, byrow=TRUE),
						matrix(c(0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0), nc=7, byrow=TRUE),
						matrix(c(0,0,1,1,0,0,1,1,0,0,0,0,0,0,0,0), nc=4, byrow=TRUE),
						matrix(c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,1,0,0,0), nc=7, byrow=FALSE)
					)
					thetaList <- list(
						rep(0.5, 4),
						rep(0.5, 7),
						c(0.80, 0.75, 0.67, 0.5),
						c(0.8750, 0.8571, 0.8333, 0.8000, 0.7500, 0.6667, 0.5000),
						c(0.8, 0.8, 0.5, 0.5),
						c(0.4201, 0.3965, 0.1770, 0.5929, 0.6881, 0.8745, 0.6539)
					)
					epsilonList <- list(0.01, 0.001, 0.1)
					sigmaList <- list(1, 0.1)
					sampleSizeList <- c(500, 1000)
					
				## Get poset
					simPars <- list()
					simPars$choice <- paste(c("P", "E", "S", "N"), c(nP, nE, nS, nN), sep="", collapse="_")
					simPars$P <- posetList[[nP]]
					p <- ncol(simPars$P)
					simPars$H <- hyperCube(p)
					simPars$G <- orderIdeals(simPars$H, simPars$P)$G
					simPars$thetas <- thetaList[[nP]]
					if (nP==5){ ## For toy poset, draw thetas randomly
						simPars$thetas <- generateThetas(p)
					}
					simPars$epsilon <- epsilonList[[nE]]
					simPars$sigma2 <- sigmaList[[nS]]
					simPars$nGen <- sampleSizeList[[nN]]
					simPars$mus <- generateMus(simPars$P, -1, 3)
					simPars$betas <- NULL
					cat("Choice is:", simPars$choice, "\n")
					printParams(simPars, "Parameters used for simulation")
				
				## Set vertex names
					if (nP != 6){
						eventNames <- paste("Event", seq(1:p), sep="")
					} else {
						eventNames <- c("41L", "67N", "69D", "70R", "210W", "215Y", "219Q")	
					}
					
				## Simulate data from the generative model
					cbnData <- generateData(simPars$P, simPars$thetas, simPars$epsilon, 
                                  simPars$mus, simPars$sigma2, simPars$nGen,
                                  simPars$betas, isoReg=TRUE)
					yGen <- cbnData[,1]
					xGen <- cbnData[,c(2:(nrow(simPars$P)+1))]
					zGen <- cbnData[,c((nrow(simPars$P)+2):(2*nrow(simPars$P)+1))]
					colnames(xGen) <- eventNames
					zStat <- table(apply(zGen, 1, function(x){ paste(x, collapse="") }))
					xStat <- table(apply(xGen, 1, function(x){ paste(x, collapse="") }))
				
				##	Determine poset by annealing or use given predefined poset
					if (structureSearch){
            simPars$pUse <- doAnneal(xGen, yGen, outputPath, tInit=tInit, nIter=nIter, loggingFlag=FALSE, tAlpha=tAlpha, 
															       epsStop=epsStop, epsGuess=epsGuess, tFactor=tFactor, maxIter=maxIter)$P
          } else {
            simPars$pUse <- simPars$P
					}
					
				## Assess performance
					cvResult <- cvIcbn(fitCbn, xGen, yGen, K, simPars$pUse, epsStop=epsStop)
					pValMse <- wilcox.test(cvResult$mseAllLr, cvResult$mseAllIcbn, , alternative="greater", paired=FALSE)$p.value
					pValCor <- wilcox.test(cvResult$corAllLr, cvResult$corAllIcbn, alternative="less", paired=FALSE)$p.value
					pValMseP <- wilcox.test(cvResult$mseAllLr, cvResult$mseAllIcbn, alternative="greater", paired=TRUE)$p.value
					pValCorP <- wilcox.test(cvResult$corAllLr, cvResult$corAllIcbn, alternative="less", paired=TRUE)$p.value
					results <- list(cvResult=cvResult, xStat=xStat, zStat=zStat, simPars=simPars, xGen=xGen, yGen=yGen, zGen=zGen,
														pValMse=pValMse, pValCor=pValCor, pValMseP=pValMseP, pValCorP=pValCorP)
					save(results, file=paste(outputPath, "results_", simPars$choice, "_", run, ".Rdata", sep=""))
					
				## Have a look at the model
					idxTrain <- sample(1:nrow(xGen), ceil(nrow(xGen)*9/10))
					idxTest <- setdiff(1:nrow(xGen), idxTrain)
					xTrain <- xGen[idxTrain,]
					xTest <- xGen[idxTest, ]
					yTrain <- yGen[idxTrain]
					yTest <- yGen[idxTest]
					
				## Train model using the true/fitted poset
					parIcbn <- guessParams(xTrain, yTrain, simPars$pUse)
					parIcbn <- doEm(xTrain, yTrain, parIcbn, eps=epsStop, maxIter=maxIter)
					printParams(simPars, "Parameters used for simulation")
					printParams(parIcbn, "Fitted parameters on 90% of the data")
					
				## Predict test data
					parLr		<- lm(yTrain~as.matrix(xTrain))$coef
					perfTest <- predictPheno(xTest, yTest, parIcbn, parLr)
						
				## Plot both models, simulated and fitted
					pdf(file=paste(outputPath, "plots_", simPars$choice, "_", run, ".pdf", sep=""), width=12, height=9)
						par(mfrow=c(2,2), cex.sub=0.7, cex.main=0.8, cex.lab=0.7)
						ug <- unlist(zStat)
						plotPoset(simPars$P, vertexLabels=paste(eventNames, round(simPars$thetas, 2), sep="\n"), 
							main=paste("Simulation: ", simPars$choice, "\nrun: ", run, ", sample size: ", simPars$nGen, sep=""),
											sub=paste("Epsilon: ", simPars$epsilon, ", Sigma2: ", simPars$sigma2, 
											"\nVertex labels: Effective thetas used for simulation",
											"\nEffective mus: ", paste(round(simPars$mus,2), collapse=", "),
											sep=""), vertex.size=40, edge.arrow.size=0.2, vertex.label.cex=0.6)
						plotIsotonic(simPars$mus, simPars$G, simPars$H, colnames(xGen), ug, layout="reingold",
											sub="Genotype lattice used for simulation\nVertex labels: genotype, effective count, mu at genotype.", 
											vertex.size=40, edge.arrow.size=0.2, vertex.label.cex=0.6)
						ug <- round(colSums(parIcbn$respMat),0)
						plotPoset(simPars$pUse, vertexLabels=paste(colnames(xTrain), round(parIcbn$thetas, 2), sep="\n"), main=paste("lObs:", round(parIcbn$lObs,2), sep=""),
											sub=paste("Epsilon: ", round(parIcbn$epsilon,2), ", Sigma2: ", round(parIcbn$sigma2,2), 
											"\nVertex labels: Fitted thetas",
											"\npVal(MSE): ", round(pValMse, 4), ", pVal(COR): ", round(pValCor, 4),	
											"\npVal(MSE) paired: ", round(pValMseP, 4), ", pVal(COR) paired: ", round(pValCorP, 4),
											sep=""), vertex.size=40, edge.arrow.size=0.2, vertex.label.cex=0.6)
						plotIsotonic(parIcbn$mus, parIcbn$G, parIcbn$H, colnames(xTrain), ug, layout="reingold", 
											sub=paste(
											K, "-fold cross-validation\nVertex labels: genotype, fitted expected count, fitted mu at genotype.\n",
											"MSE(LR): ", round(cvResult$mseLr, 2), "(", round(cvResult$seMseLr, 3), ")", 
											", MSE(CBN): ", round(cvResult$mseIcbn, 2), "(", round(cvResult$seMseIcbn, 3), ")\n",
											"COR(LR): ", round(cvResult$corLr, 2), "(", round(cvResult$seCorLr, 3), ")",
											", COR(CBN): ", round(cvResult$corIcbn, 2), "(", round(cvResult$seCorIcbn, 3), ")\n",
											sep=""), vertex.size=40, edge.arrow.size=0.2, vertex.label.cex=0.6)
						plot(perfTest$y, perfTest$yhatIcbn, col="green", xlab="Observed value", ylab="Predicted value", 
							main=paste("Predicted vs. observed y-values\nSimulation: ", simPars$choice, sep=""), pch=16)
						points(perfTest$y, perfTest$yhatLr, col="red", pch=17)
						abline(0,1)
						legend("topleft", c("Icbn", "Lr"), col=c("green", "red"), pch=c(16,17), cex=0.7)
						plot(1:length(perfTest$y), (perfTest$y), xlab="Index", ylab="Predicted value", main="y-values in data order", pch=18, col="black")
						points(1:length(perfTest$y), (perfTest$yhatLr), col="red", pch=17)
						points(1:length(perfTest$y), (perfTest$yhatIcbn), col="green", pch=16)
						plot(sort(yGen), main="Sorted generated y-values", sub="horizontal lines correspond\nto the mus used for generation", xlab="",
										ylim=c(-2, 4), ylab="")
						abline(h=simPars$mus, lty=2)
						plot(sort(perfTest$y), col="black", main="fitted values", sub="horizontal lines correspond\nto the mus fitted", pch=18, xlab="",
										ylim=c(-2, 4), ylab="")
						points(perfTest$yhatLr[order(perfTest$y)], col="red", pch=17)
						points(perfTest$yhatIcbn[order(perfTest$y)], col="green", pch=16)
						abline(h=parIcbn$mus, lty=2)
						legend("bottomright", c("True", "Icbn", "Lr"), col=c("black", "green", "red"), pch=c(18, 16, 17), cex=0.7)
					dev.off()
					} 
				}
			}
		}
	
## End
